import json
from jnius import autoclass


def load_java_tp(tp_json):
    """Just used for testing, users won't need this."""
    string_java = autoclass("java.lang.String")
    java_transform_process = autoclass("org.datavec.api.transform.TransformProcess")
    java_transform_process.fromJson(string_java(tp_json))


def inference_from_json(as_json):
    string_java = autoclass("java.lang.String")
    inference_configuration_java = autoclass(
        "ai.konduit.serving.InferenceConfiguration"
    )
    inference_configuration_java.fromJson(string_java(json.dumps(as_json)))



import json
import pydatavec
import pytest
from konduit import *
from konduit.json_utils import config_to_dict_with_type
from konduit.server import Server
from konduit.utils import is_port_in_use

from .utils import load_java_tp, inference_from_json


@pytest.mark.integration
@pytest.mark.parametrize("output_format", ["ARROW", "JSON"])
def test_build_tp(output_format):
    schema = pydatavec.Schema()
    schema.add_string_column("first")
    tp = pydatavec.TransformProcess(schema)
    tp.append_string("first", "two")
    java_tp = tp.to_java()

    tp_json = java_tp.toJson()
    load_java_tp(tp_json)
    _ = json.dumps(tp_json)

    as_python_json = json.loads(tp_json)
    transform_process = (
        TransformProcessStep()
        .set_input(column_names=["first"], types=["String"])
        .set_output(column_names=["first"], types=["String"])
        .transform_process(as_python_json)
    )

    serving_config = ServingConfig(
        output_data_format=output_format,
        log_timings=True,
    )

    inference_config = InferenceConfiguration(
        serving_config=serving_config, steps=[transform_process]
    )
    as_json = config_to_dict_with_type(inference_config)
    inference_from_json(as_json)

    server = Server(
        inference_config=inference_config,
        extra_start_args="-Xmx8g",
        jar_path="konduit.jar",
    )
    _, port, started = server.start()

    assert started
    assert is_port_in_use(port)

    client = server.get_client()

    try:
        predicted = client.predict({"first": "value"})
        print(predicted)
        server.stop()
    except Exception as e:
        print(e)
        server.stop()



import json
import logging
import sys
import os
import re
import requests
import signal
import subprocess
import uuid
from konduit import KONDUIT_JAR_DIR
from konduit.base_inference import PipelineStep
from konduit.client import Client
from konduit.inference import InferenceConfiguration
from konduit.json_utils import config_to_dict_with_type


def stop_server_by_pid(pid):
    """Stop a Konduit server with a given process ID

    :param pid: process ID of a running Konduit Server
    """
    if pid:
        os.kill(pid, signal.SIGTERM)


class Server(object):
    def __init__(
        self,
        inference_config=None,
        serving_config=None,
        steps=None,
        extra_start_args="-Xmx8g",
        extra_jar_args=None,
        config_path="config.json",
        jar_path=None,
        pid_file_path="konduit-serving.pid"
    ):
        """Konduit Server

        Start and stop a server from a given inference configuration. Alternatively,
        you can provide a serving configuration and a list of steps, instead of an
        inference config.
        Extra arguments with JVM options can be passed.

        Example:

        >>> import konduit
        >>> server = konduit.Server()

        :param inference_config: InferenceConfiguration instance
        :param serving_config: ServingConfig instance
        :param steps: list (or single) PipelineSteps
        :param extra_start_args: list of string arguments to start the process with
        :param extra_jar_args: extra jar files to add to the classpath
        :param config_path: path to write the config object to (as json)
        :param jar_path: path to the konduit uberjar
        """
        self.port = -1
        if jar_path is None:
            jar_path = os.getenv(
                "KONDUIT_JAR_PATH", os.path.join(KONDUIT_JAR_DIR, "konduit.jar")
            )

        if inference_config:
            self.config = inference_config
        elif serving_config and steps:
            if isinstance(steps, PipelineStep):
                steps = [steps]
            self.config = InferenceConfiguration(
                steps=steps, serving_config=serving_config
            )
        else:
            self.config = InferenceConfiguration()
        self.config_path = config_path
        self.jar_path = jar_path
        self.pid_file_path = pid_file_path
        self.process = None
        if extra_start_args is None:
            extra_start_args = []

        if extra_jar_args is None:
            extra_jar_args = []

        # Handle singular element case
        if type(extra_start_args) is not list:
            self.extra_start_args = [extra_start_args]
        else:
            self.extra_start_args = extra_start_args

        if type(extra_jar_args) is not list:
            self.extra_jar_args = [extra_jar_args]
        else:
            self.extra_jar_args = extra_jar_args

        self.server_id = uuid.uuid4().hex[:8]

    def get_client(self,
                   input_data_format=None,
                   prediction_type=None,
                   output_data_format=None):
        """Get a Konduit Client instance from this Server instance.
        :param output_data_format: optional, same as in Client signature
        :return: konduit.Client
        """
        serving_config = self.config._get_serving_config()
        steps = self.config._get_steps()
        input_names = []
        output_names = []
        for step in steps:
            input_names += step._get_input_names()
            output_names += step._get_output_names()

        port = self.port
        host = serving_config._get_listen_host()
        if not host:
            host = "http://localhost"
        else:
            if not host.startswith("http://"):
                host = "http://" + host

        if not input_data_format:
            input_data_format = "NUMPY"

        if not prediction_type:
            prediction_type = "RAW"

        if not output_data_format:
            output_data_format = serving_config._get_output_data_format()

        return Client(
            host=host,
            port=port,
            input_data_format=input_data_format,
            output_data_format=output_data_format,
            prediction_type=prediction_type,
            input_names=input_names,
            output_names=output_names,
        )

    def start(self, server_id=None):
        """Start the Konduit server

        :param server_id the server will be started with this id.
        """

        if not server_id:
            server_id = self.server_id
        else:
            self.server_id = server_id

        json_config = config_to_dict_with_type(self.config)
        with open(self.config_path, "w") as f:
            abs_path = os.path.abspath(self.config_path)
            logging.info("Wrote config.json to path " + abs_path)
            json.dump(json_config, f)

        args = self._process_extra_args(abs_path, server_id)
        process = subprocess.Popen(args=args, stdout=subprocess.PIPE)
        self.process = process

        # Check if the server is up or not.
        request_timeout = 5

        started = False

        print("Starting server")

        port = -1
        while True:
            output = process.stdout.readline()
            if output == b'' and process.poll() is not None:
                break
            else:
                print(output.decode("utf-8"), end="")
                if b'Inference server started on port' in output:
                    m = re.search('port (.+?) with', str(output))
                    if m:
                        port = int(m.group(1))
                        self.port = port
                        break

        if port != -1:
            try:
                r = requests.get(
                    "http://localhost:{}/healthcheck".format(port),
                    timeout=request_timeout,
                )
                if r.status_code == 204:
                    started = True
            except Exception as ex:
                logging.debug("Server health checks failed...\n{}".format(str(ex)))

        process = process.poll()

        if started:
            print("\n\nServer has started successfully on port {}".format(port))
        else:
            print("\n\nThe server wasn't able to start.")
            self.stop()

        return process, port, started

    def stop(self, server_id=None):
        """Stop the server"""
        if not server_id:
            server_id = self.server_id
        else:
            self.server_id = server_id

        command = self.get_command("stop " + server_id)
        logging.info("Running with args\n" + " ".join(command))
        subprocess.call(command, shell=sys.platform.startswith("win"))

    def _process_extra_args(self, absolute_path, server_id):
        """Process submitted extra arguments list.

        :param absolute_path: absolute path of the configuration file
        :return: concatenated string arguments
        """
        args = self.get_command("serve -c " + absolute_path)

        if server_id:
            args.append("-id")
            args.append(server_id)

        logging.info("Running with args\n" + " ".join(args))
        return args

    def get_command(self, command):
        classpath = [self.jar_path]
        classpath.extend(self.extra_jar_args)

        args = ["java"]
        # Pass extra jvm arguments such as memory.
        if self.extra_start_args:
            args.extend(self.extra_start_args)
        args.append("-cp")
        args.append(os.pathsep.join(classpath))
        args.append("ai.konduit.serving.launcher.KonduitServingLauncher")
        if command:
            args.extend(command.strip().split(" "))
        return args



import os

USER_PATH = os.path.expanduser("~")
KONDUIT_BASE_DIR = os.path.join(USER_PATH, ".konduit-serving")
KONDUIT_JAR_DIR = os.path.join(KONDUIT_BASE_DIR, "jar")

jar = os.getenv("KONDUIT_JAR_PATH", os.path.join(KONDUIT_JAR_DIR, "konduit.jar"))

try:
    import pydl4j

    pydl4j.add_classpath(jar)
except Exception as e:
    print("VM already running from previous test")
    print(e)

from .inference import *
from .server import *
from .client import *



from jnius import autoclass
from konduit.base_inference import *

try:
    SchemaTypeUtils = autoclass("ai.konduit.serving.util.SchemaTypeUtils")
except Exception as e:
    raise Exception(
        "We couldn't initialize the Java classes used by Konduit. "
        "Make sure to have a konduit.jar available on your PATH, "
        "for instance by setting the environment variable KONDUIT_JAR_PATH "
        "to point to such a JAR.\n",
        e,
    )


def set_input_columns_func(self, column_names, input_name="default"):
    input_columns = self._get_input_column_names()
    if input_columns is None:
        input_columns = {}
    input_columns[input_name] = column_names
    self._set_input_column_names(input_columns)


def set_output_columns_func(self, column_names, output_name="default"):
    output_columns = self._get_output_column_names()
    if output_columns is None:
        output_columns = {}
    output_columns[output_name] = column_names
    self._set_output_column_names(output_columns)


def set_input_types_func(self, types, input_name="default"):
    schemas = self._get_input_schemas()
    if schemas is None:
        schemas = {}
    schemas[input_name] = types
    self._set_input_schemas(schemas)


def set_output_types_func(self, types, output_name="default"):
    schemas = self._get_output_schemas()
    if schemas is None:
        schemas = {}
    schemas[output_name] = types
    self._set_output_schemas(schemas)


def set_input_func(
    self, schema=None, column_names=None, types=None, input_name="default"
):
    names = self._get_input_names()
    if names is None:
        names = []
    if input_name not in names:
        if schema is not None:
            column_names = SchemaTypeUtils.columnNames(schema)
            types = SchemaTypeUtils.typesForSchema(schema)
        if column_names is None or types is None:
            raise Exception(
                "Please provide either an input schema, or column names and types"
            )
        names.append(input_name)
        self._set_input_names(names)

        self.set_input_columns(input_name=input_name, column_names=column_names)
        self.set_input_types(input_name=input_name, types=types)

        return self
    else:
        raise Exception(
            "Input name"
            + input_name
            + " is already configured for this PipelineStep, "
            + "please choose another name for your next step."
        )


def set_output_func(
    self, schema=None, column_names=None, types=None, output_name="default"
):
    names = self._get_output_names()
    if names is None:
        names = []
    if output_name not in names:
        if schema is not None:
            column_names = SchemaTypeUtils.columnNames(schema)
            types = SchemaTypeUtils.typesForSchema(schema)
        if column_names is None or types is None:
            raise Exception(
                "Please provide either an output schema, or column names and types"
            )
        names.append(output_name)
        self._set_output_names(names)

        self.set_output_columns(output_name=output_name, column_names=column_names)
        self.set_output_types(output_name=output_name, types=types)

        return self
    else:
        raise Exception(
            "Input name"
            + output_name
            + " is already configured for this PipelineStep, "
            + "please choose another name for your next step."
        )


PipelineStep.set_input_columns = set_input_columns_func
PipelineStep.set_output_columns = set_output_columns_func
PipelineStep.set_input_types = set_input_types_func
PipelineStep.set_output_types = set_output_types_func
PipelineStep.set_input = set_input_func
PipelineStep.set_output = set_output_func


def step_func(
    self, input_schema, output_schema, transform_process, input_name="default"
):
    self.set_input(input_name=input_name, schema=input_schema)
    self.set_output(output_name=input_name, schema=output_schema)
    self.transform_process(input_name=input_name, transform_process=transform_process)

    return self


def transform_process_func(self, transform_process, input_name="default"):
    tps = self._get_transform_processes()
    if tps is None:
        tps = {}
    tps[input_name] = transform_process
    self._set_transform_processes(tps)

    return self


TransformProcessStep.step = step_func
TransformProcessStep.transform_process = transform_process_func


def python_step_func(
    self,
    python_config,
    input_name="default",
    input_schema=None,
    input_column_names=None,
    input_types=None,
    output_schema=None,
    output_column_names=None,
    output_types=None,
):
    if not input_name: 
        raise TypeError("input_name must not be None or empty string")

    # if nothing else is defined, we can derive all properties just from the Python configuration
    if (
        input_schema is None
        and input_column_names is None
        and input_types is None
        and output_schema is None
        and output_column_names is None
        and output_types is None
    ):
        inputs = python_config._get_python_inputs()
        outputs = python_config._get_python_outputs()
        if isinstance(inputs, list):
            input_column_names = inputs
            input_types = ["NDArray" for i in inputs]
        else:
            input_column_names = list(inputs.keys())
            input_types = [konduit_type_mapping(v) for v in inputs.values()]
        if isinstance(outputs, list):
            output_column_names = outputs
            output_types = ["NDArray" for i in outputs]
        else:
            output_column_names = list(outputs.keys())
            output_types = [konduit_type_mapping(v) for v in outputs.values()]

    self.set_input(
        schema=input_schema,
        column_names=input_column_names,
        types=input_types,
        input_name=input_name,
    )
    self.set_output(
        schema=output_schema,
        column_names=output_column_names,
        types=output_types,
        output_name=input_name,
    )
    self.python_config(python_config=python_config, input_name=input_name)

    return self


def python_config_func(self, python_config, input_name="default"):
    python_configs = self._get_python_configs()
    if python_configs is None:
        python_configs = {}
    python_configs[input_name] = python_config
    self._set_python_configs(python_configs)

    return self


def konduit_type_mapping(name):
    type_map = {
        "BOOL": "Boolean",
        "STR": "String",
        "INT": "Integer",
        "FLOAT": "Float",
        "NDARRAY": "NDArray",
    }
    if name not in type_map.keys():
        raise Exception("Can't convert input type " + str(name))
    else:
        return type_map.get(name)


PythonStep.step = python_step_func
PythonStep.python_config = python_config_func



#  ******************************************************************************
#  Copyright (c) 2020 Konduit K.K.
#
#  This program and the accompanying materials are made available under the
#  terms of the Apache License, Version 2.0 which is available at
#  https://www.apache.org/licenses/LICENSE-2.0.
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
#
#  SPDX-License-Identifier: Apache-2.0
#  *****************************************************************************

import os
import sys
import subprocess
import requests
from hurry.filesize import size
from packaging.version import parse
import click

USER_PATH = os.path.expanduser("~")
KONDUIT_BASE_DIR = os.path.join(USER_PATH, ".konduit-serving")
KONDUIT_SOURCE_DIR = os.path.join(KONDUIT_BASE_DIR, "source")
KONDUIT_JAR_DIR = os.path.join(KONDUIT_BASE_DIR, "jar")
KONDUIT_JAR_PATH = os.path.join(KONDUIT_JAR_DIR, "konduit.jar")

INCOMPATIBLE_COMPILATION_TAGS = ["cli_base", "cli_base_2", "cli_base_3", "cli_base_4", "cli_base_5"]
DOWNLOAD_TAG = "cli_base"

LAST_COMPATIBLE_KONDUIT_VERSION = "0.1.0-SNAPSHOT"
DEFAULT_KONDUIT_TAG = "cli_base_5"
KONDUIT_JAR_URL_FORMAT = "https://github.com/KonduitAI/konduit-serving/releases/download/" \
                         "{tag}/konduit-serving-uberjar-{version}-{spin}-{platform}-{chip}${cuda_version}.jar"

os_choices = [
    "windows-x86_64",
    "linux-x86_64",
    "macosx-x86_64",
    "linux-armhf",
]


def get_platform():
    if sys.platform.startswith("win32"):
        return "windows-x86_64"
    elif sys.platform.startswith("darwin"):
        return "macosx-x86_64"
    elif sys.platform.startswith("linux"):
        return "linux-x86_64"
    else:
        raise RuntimeError("Please specify '--os'. Possible values are: " + os_choices.__str__())


def download_if_required(url, save_path):
    if not os.path.exists(KONDUIT_JAR_DIR):
        os.makedirs(KONDUIT_JAR_DIR)

    response = requests.get(url, stream=True)
    status_code = response.status_code
    if status_code != 200:
        print("Download path: {}".format(url))
        print("Failed with a status code of {}".format(status_code))
        return

    total = response.headers.get('content-length')

    if os.path.exists(save_path) and os.path.getsize(KONDUIT_JAR_PATH) == total:
        print("The required CLI binary has already been downloaded.")
        return
    else:
        print("Downloading command line binaries from " + url)

    with open(save_path, 'wb') as f:
        if total is None:
            f.write(response.content)
        else:
            downloaded = 0
            total = int(total)
            for data in response.iter_content(chunk_size=max(int(total / 1000), 1024 * 1024)):
                downloaded += len(data)
                f.write(data)
                done = int(50 * downloaded / total)
                sys.stdout.write('\r[{}{}]'.format('█' * done, '.' * (50 - done)) +
                                 (" ({}/{})".format(size(downloaded), size(total))))
                sys.stdout.flush()
    sys.stdout.write('\n')


def git_clone_konduit(use_https=True, tag=DEFAULT_KONDUIT_TAG):
    """Clone the konduit-serving git repo, if it doesn't already exist locally."""

    if not os.path.exists(KONDUIT_SOURCE_DIR):
        os.makedirs(KONDUIT_SOURCE_DIR)

    if use_https:
        repo = "https://github.com/KonduitAI/konduit-serving.git"
    else:
        repo = "git@github.com:KonduitAI/konduit-serving.git"
    try:
        if not os.listdir(KONDUIT_SOURCE_DIR):
            subprocess.call(["git", "clone", repo, KONDUIT_SOURCE_DIR],
                            shell=sys.platform.startswith("win"))
        # Pulling in changes if needed
        subprocess.call(["git", "-C", KONDUIT_SOURCE_DIR, "fetch", "--all"])
        subprocess.call(["git", "checkout", tag], cwd=KONDUIT_SOURCE_DIR,
                        shell=sys.platform.startswith("win"))
        subprocess.call(["git", "-C", KONDUIT_SOURCE_DIR, "pull"])
    except Exception as e:
        raise RuntimeError(">>> Could not clone konduit-serving repository and switch to commit hash {}"
                           "Make sure to have git installed. Type 'konduit-init --help' for help .\n"
                           .format(tag), e)


def build_jar(operating_sys, spin, chip, cuda_version, options):
    """Build the actual JAR, using our mvnw wrapper under the hood."""

    if not os.path.exists(KONDUIT_JAR_DIR):
        os.makedirs(KONDUIT_JAR_DIR)

    if operating_sys is None:
        operating_sys = get_platform()

    # Building the uber-jar file
    try:
        command = [
            "python",
            os.path.join(KONDUIT_SOURCE_DIR, "build_jar.py"),
            "--os",
            operating_sys,
            "--source",
            KONDUIT_SOURCE_DIR,
            "--spin",
            spin,
            "--chip",
            chip,
            "--cuda-version",
            cuda_version,
            "--target",
            KONDUIT_JAR_PATH
        ]
        command.extend(["--option {}".format(option) for option in options])

        subprocess.call(
            command,
            shell=sys.platform.startswith("win")
        )
    except Exception as e:
        RuntimeError("Failed to build jar.\n", e)


def get_git_tags():
    response = requests.get("https://api.github.com/repos/KonduitAI/konduit-serving/tags")
    status_code = response.status_code

    if status_code == 200:
        tags = response.json()
        return [tag['name'] for tag in tags]
    else:
        return [DEFAULT_KONDUIT_TAG]


def get_jar_url(platform, version, spin, chip, cuda_version=None):
    if not platform:
        platform = get_platform()

    return KONDUIT_JAR_URL_FORMAT.format(version=version,
                                         platform=platform,
                                         tag=DOWNLOAD_TAG,
                                         spin=spin,
                                         chip=chip,
                                         cuda_version=("-cuda" + cuda_version) if chip == "gpu" else "")


git_tags = list(set(get_git_tags()).difference(INCOMPATIBLE_COMPILATION_TAGS))
if len(git_tags) == 0:
    git_tags = ["master"]
DEFAULT_KONDUIT_TAG = git_tags[0]  # Assuming the first one in the response is the most recent one


@click.command()
@click.option(
    "-p", "--platform",
    help="Your operating system. "
         "Defaults to the cpu version of the "
         "current OS platform in use.",
    type=click.Choice(os_choices),
)
@click.option(
    "--https",
    type=bool,
    default=True,
    show_default=True,
    help="If True, use HTTPS to clone konduit-serving, else SSH."
)
@click.option(
    "-t", "--tag",
    default=DEFAULT_KONDUIT_TAG,
    show_default=True,
    help="The git tag to build the source code from.",
    type=click.Choice(git_tags)
)
@click.option(
    "-s", "--spin",
    default="all",
    show_default=True,
    help="Whether to bundle Python ('python'), PMML ('pmml'), both ('all') " +
         "or neither ('minimal'). Python bundling is not encouraged with ARM, " +
         "and PMML bundling is not encouraged if agpl license is an issue."
)
@click.option(
    "-c", "--chip",
    default="cpu",
    show_default=True,
    help="Specifies the chip architecture which could be cpu, gpu (CUDA) or arm.",
    type=click.Choice(["cpu", "gpu", "arm"])
)
@click.option(
    "-cv", "--cuda-version",
    default="10.2",
    show_default=True,
    help="Specifies the CUDA version to compile when `--chip == gpu`",
    type=click.Choice(["10.0", "10.1", "10.2"])
)
@click.option(
    "-o", "--options",
    multiple=True,
    help="Extra java property to add to the build command. Can be used multiple times."
         "Format: -o <property1.name>=<property1.value>"
)
@click.option(
    "-v", "--version",
    default=LAST_COMPATIBLE_KONDUIT_VERSION,
    show_default=True,
    help="Only works with the `--download` option to specify the version of konduit-serving to be downloaded."
)
@click.option(
    "-d", "--download",
    is_flag=True,
    help="If set, downloads the pre-built jar file instead of building it. "
         "Also works with `--platform`, `--chip`, `--cuda-version`, `--version` and `--spin` options."
)
def init(platform, https, tag, spin, chip, cuda_version, options, version, download):
    if download:
        if parse(LAST_COMPATIBLE_KONDUIT_VERSION.replace("-SNAPSHOT", "")) < parse(version.replace("-SNAPSHOT", "")):
            print("This version of Python CLI is only compatible with versions <= {}"
                  .format(LAST_COMPATIBLE_KONDUIT_VERSION))
        else:
            download_if_required(get_jar_url(platform, version, spin, chip, cuda_version), KONDUIT_JAR_PATH)
    else:
        git_clone_konduit(https, tag)
        build_jar(platform, spin, chip, cuda_version, options)



import os
import pytest
from konduit.utils import *


@pytest.mark.unit
def test_unix_replacement():
    file_path = "C:\\foo\\bar;baz"
    unix_path = to_unix_path(file_path)
    assert unix_path == "C:/foo/bar{}baz".format(os.pathsep)

    step_config = {"python_path": file_path, "bar": 42, "keep_this": file_path}
    unix_step_config = update_dict_with_unix_paths(step_config)
    assert unix_step_config["python_path"] == "C:/foo/bar{}baz".format(os.pathsep)
    assert unix_step_config["bar"] == 42
    assert unix_step_config["keep_this"] == file_path



import numpy as np
import pytest
import random
from konduit import *
from konduit.server import Server
from konduit.utils import is_port_in_use


@pytest.mark.integration
def test_server_start():
    port = random.randint(1000, 65535)
    serving_config = ServingConfig(http_port=port)

    python_config = PythonConfig(
        python_code="first += 2",
        python_inputs={"first": "NDARRAY"},
        python_outputs={"first": "NDARRAY"},
    )

    step = PythonStep().step(python_config)
    server = Server(steps=step, serving_config=serving_config)
    server.start()

    client = server.get_client()

    data_input = {"default": np.load("../data/input-0.npy")}

    assert is_port_in_use(port)

    try:
        predicted = client.predict(data_input)
        print(predicted)
        server.stop()
    except Exception as e:
        print(e)
        server.stop()



import numpy as np
import os
import pytest
from konduit import *
from konduit.client import Client
from konduit.server import Server
from konduit.utils import is_port_in_use, default_python_path


@pytest.mark.integration
def test_python_script_prediction():
    work_dir = os.path.abspath(".")

    python_config = PythonConfig(
        python_path=default_python_path(work_dir),
        python_code_path=os.path.join(work_dir, "simple.py"),
        python_inputs={"first": "NDARRAY"},
        python_outputs={"second": "NDARRAY"},
    )

    step = PythonStep().step(python_config)
    server = Server(steps=step, serving_config=ServingConfig())
    _, port, started = server.start()

    assert started
    assert is_port_in_use(port)

    client = Client(port=port)

    try:
        input_array = np.load("../data/input-0.npy")
        predicted = client.predict(input_array)
        print(predicted)
        server.stop()
    except Exception as e:
        print(e)
        server.stop()
